@@ -9,39 +9,61 @@ import functools
 import hashlib
 import threading
 
-from lib.core.settings import MAX_CACHE_ITEMS
 from lib.core.datatype import LRUDict
+from lib.core.settings import MAX_CACHE_ITEMS
+from lib.core.settings import UNICODE_ENCODING
 from lib.core.threads import getCurrentThreadData
 
-_lock = threading.Lock()
+_cache = {}
+_cache_lock = threading.Lock()
+_method_locks = {}
 
-def cachedmethod(f, cache=LRUDict(capacity=MAX_CACHE_ITEMS)):
+def cachedmethod(f):
     """
     Method with a cached content
 
+    >>> __ = cachedmethod(lambda _: _)
+    >>> __(1)
+    1
+    >>> __ = cachedmethod(lambda *args, **kwargs: args[0])
+    >>> __(2)
+    2
+    >>> __ = cachedmethod(lambda *args, **kwargs: list(kwargs.values())[0])
+    >>> __(foobar=3)
+    3
+
     Reference: http://code.activestate.com/recipes/325205-cache-decorator-in-python-24/
     """
 
+    _cache[f] = LRUDict(capacity=MAX_CACHE_ITEMS)
+
     @functools.wraps(f)
-    def _(*args, **kwargs):
-        key = int(hashlib.md5("|".join(str(_) for _ in (f, args, kwargs))).hexdigest(), 16) & 0x7fffffffffffffff
+    def _f(*args, **kwargs):
+        key = int(hashlib.md5("|".join(str(_) for _ in (f, args, kwargs)).encode(UNICODE_ENCODING)).hexdigest(), 16) & 0x7fffffffffffffff
 
         try:
-            with _lock:
-                result = cache[key]
+            with _cache_lock:
+                result = _cache[f][key]
         except KeyError:
             result = f(*args, **kwargs)
 
-            with _lock:
-                cache[key] = result
+            with _cache_lock:
+                _cache[f][key] = result
 
         return result
 
-    return _
+    return _f
 
 def stackedmethod(f):
     """
     Method using pushValue/popValue functions (fallback function for stack realignment)
+
+    >>> threadData = getCurrentThreadData()
+    >>> original = len(threadData.valueStack)
+    >>> __ = stackedmethod(lambda _: threadData.valueStack.append(_))
+    >>> __(1)
+    >>> len(threadData.valueStack) == original
+    True
     """
 
     @functools.wraps(f)
@@ -58,3 +80,16 @@ def stackedmethod(f):
         return result
 
     return _
+
+def lockedmethod(f):
+    @functools.wraps(f)
+    def _(*args, **kwargs):
+        if f not in _method_locks:
+            _method_locks[f] = threading.RLock()
+
+        with _method_locks[f]:
+            result = f(*args, **kwargs)
+
+        return result
+
+    return _
