@@ -6,15 +6,11 @@ See the file 'LICENSE' for copying permission
 """
 
 from lib.core.common import Backend
-from lib.core.data import conf
 from lib.core.datatype import AttribDict
 from lib.core.settings import EXCLUDE_UNESCAPE
 
 class Unescaper(AttribDict):
     def escape(self, expression, quote=True, dbms=None):
-        if conf.noEscape:
-            return expression
-
         if expression is None:
             return expression
 
