@@ -9,11 +9,14 @@ from lib.core.data import logger
 from plugins.generic.enumeration import Enumeration as GenericEnumeration
 
 class Enumeration(GenericEnumeration):
-    def __init__(self):
-        GenericEnumeration.__init__(self)
-
     def getPasswordHashes(self):
         warnMsg = "on DB2 it is not possible to list password hashes"
         logger.warn(warnMsg)
 
         return {}
+
+    def getStatements(self):
+        warnMsg = "on DB2 it is not possible to enumerate the SQL statements"
+        logger.warn(warnMsg)
+
+        return []
