@@ -9,9 +9,6 @@ from lib.core.data import logger
 from plugins.generic.enumeration import Enumeration as GenericEnumeration
 
 class Enumeration(GenericEnumeration):
-    def __init__(self):
-        GenericEnumeration.__init__(self)
-
     def getDbs(self):
         warnMsg = "on Firebird it is not possible to enumerate databases (use only '--tables')"
         logger.warn(warnMsg)
@@ -39,3 +36,9 @@ class Enumeration(GenericEnumeration):
     def getHostname(self):
         warnMsg = "on Firebird it is not possible to enumerate the hostname"
         logger.warn(warnMsg)
+
+    def getStatements(self):
+        warnMsg = "on Firebird it is not possible to enumerate the SQL statements"
+        logger.warn(warnMsg)
+
+        return []
